#include <iostream.h>
#include <stdlib.h>
#include <GL/glut.h>

#define MAX_VERTS 7000

extern height;

/*A structure for storing the points that make the polygon*/
typedef struct {
	GLint x,y;
}Vertex;

typedef struct tEdge {
  int yUpper;
  float xIntersect, dxPerScan;
  struct tEdge *next;
} Edge;


class Polygon{
 
public:
	Vertex verts[MAX_VERTS];
	int numVerts;
    
	Polygon();	/*Constructor*/
   ~Polygon();	/*Constructor*/
    void scanFill();  
};

void insertEdge(Edge *list, Edge *edge) {	
  Edge *p, *q = list;

  p = q->next;
  while(p != NULL) {
    if(edge->xIntersect < p->xIntersect)
      p = NULL;
    else {
      q = p;
      p = p->next;
    }
  }
  edge->next = q ->next;
  q->next = edge;
}

int yNext(int k, int cnt,Vertex *pts){
	int j;

	if( (k+1) > (cnt -1))
		j=0;
	else
		j=k+1;
	while(pts[k].y == pts[j].y)
		if((j+1) > (cnt-1))
			j=0;
		else
			j++;
	return pts[j].y;
}

void makeEdgeRec(Vertex lower, Vertex upper, int yComp, Edge *edge, Edge *edges[]) {

	edge->dxPerScan = (float) (upper.x - lower.x)/(upper.y - lower.y);
	edge->xIntersect = lower.x;

	if(upper.y < yComp)
		edge->yUpper = upper.y - 1;
	else
		edge->yUpper = upper.y -1;	
 
	insertEdge((tEdge *)edges[lower.y], edge);
}

void buildEdgeList(int cnt, Vertex *pts, Edge *edges[]) {
  Edge *edge;
  Vertex v1, v2;
  int i, yPrev = pts[cnt-2].y;

  /* Initialize the process with the last point (connects to the first point) */
  v1.x = pts[cnt-1].x;
  v1.y = pts[cnt-1].y;

  /* for each line segment */
  for(i=0;i<cnt;i++) {

    /* end of the segment is the curernt point */
    v2 = pts[i];
    if(v1.y != v2.y) { /* not a horizontal line */

      /* create a new edge with v1.row less than v2.row */
      edge = (Edge *)malloc(sizeof(Edge));
      if(v1.y < v2.y) 
	makeEdgeRec(v1, v2, 0, edge, edges);
      else
	makeEdgeRec(v2, v1, yPrev, edge, edges);
    }
    yPrev = v1.y;
    v1 = v2;
  }

}

void buildActiveList(int scan, Edge *active, Edge *edges[]) {
  Edge *p, *q;
  p = edges[scan]->next;
  while(p) {
    q = p->next;
    insertEdge(active, p);
    p = q;
  }
}

void fillScan(int scan, Edge *active) {
  Edge *p1, *p2;
  p1 = active->next;
  while(p1) {
    p2 = p1->next;
    //for(i=p1->xIntersect;i<p2->xIntersect;i++)
	glBegin(GL_LINES);
	glVertex2f(p1->xIntersect, scan);
	glVertex2f((p2->xIntersect), scan);
	  glEnd();
//	glEnd();
	p1 = p2->next;
  }
      
}

void deleteAfter(Edge *q) {
  Edge *p = q->next;
  q->next = p->next;
  free(p);
}

void updateActiveList(int scan, Edge *active) {
  Edge *q = active, *p = active->next;

  while(p) {
    /* if the edge has ended, get rid of it */
    if(scan >= p->yUpper) {
      p = p->next;
      deleteAfter(q);
    }
    /*  otherwise, update the xIntersect value */
    else {
      p->xIntersect = p->xIntersect + p->dxPerScan;
      q = p;
      p = p->next;
    }
  }
}

void resortActiveList(Edge *active) {
  Edge *q, *p = active->next;

  active->next = NULL;
  while(p) {
    q = p->next;
    insertEdge(active, p);
    p = q;
  }
}

void fillPgon(int cnt, Vertex *pts) {
  Edge *edges[1000], *active;
  int i, scan;

  for(i=0;i<height;i++){
	  edges[i] = (Edge *) malloc (sizeof(Edge));
	  edges[i]->next = NULL;
  }
  
  buildEdgeList(cnt, pts, edges);
  active = (Edge *)malloc(sizeof(Edge));
  active->next = NULL;

  for(scan = 0; scan < height; scan++) {
    buildActiveList(scan, active, edges);
    if(active->next) {
      fillScan(scan, active);
      updateActiveList(scan, active);
      resortActiveList(active);
    }
  }

  /* free malloced edges! */
  //for(i=0;i<MAX_VERTS;i++)
    //free(edges[i]);

  //free(edges);

}


Polygon :: Polygon()
{
   numVerts = 0;
} 

Polygon::~Polygon()
{  

}
   
void Polygon::scanFill()
{ 	
	fillPgon(numVerts,verts);

}